#include "built_in.h"
#include "ACCEL_driver.h"

// ADXL345 Register Definition
#define _POWER_CTL      0x2D
#define _DATA_FORMAT    0x31
#define _BW_RATE        0x2C
#define _DATAX0         0x32
#define _DATAX1         0x33
#define _DATAY0         0x34
#define _DATAY1         0x35
#define _DATAZ0         0x36
#define _DATAZ1         0x37
#define _FIFO_CTL       0x38
#define _SPEED          0x0F          // Buffer Speed - 3200Hz

#define _ACCEL_ERROR    0x02

/**************************************************************************************************
* Function ADXL345_Write()
* -------------------------------------------------------------------------------------------------
* Overview: Function write byte of data to ADXL345
* Input: register address, data
* Output: Nothing
**************************************************************************************************/
void ADXL345_Write(unsigned short address, unsigned short data1) {
  I2C0_Enable();              // issue I2C start signal
  I2C0_Master_Slave_Addr_Set(0x1D, _I2C_DIR_MASTER_TRANSMIT);
  I2C0_Write(address,_I2C_MASTER_MODE_BURST_SEND_START);       // send byte (address of the location)
  I2C0_Write(data1,_I2C_MASTER_MODE_BURST_SEND_FINISH);         // send data (data to be written)
  I2C0_Disable();               // issue I2C stop signal
}

/**************************************************************************************************
* Function ADXL345_Read()
* -------------------------------------------------------------------------------------------------
* Overview: Function read byte of data from ADXL345
* Input: register address
* Output: data from addressed register in ADXL345
**************************************************************************************************/
unsigned short ADXL345_Read(unsigned short address) {
  unsigned short tmp = 0;
  I2C0_Enable();              // issue I2C start signal
  I2C0_Master_Slave_Addr_Set(0x1D, _I2C_DIR_MASTER_TRANSMIT);              // issue I2C start signal
  I2C0_Write(address, _I2C_MASTER_MODE_SINGLE_SEND);
  I2C0_Master_Slave_Addr_Set(0x1D, _I2C_DIR_MASTER_RECEIVE);
  I2C0_Read(&tmp, _I2C_MASTER_MODE_SINGLE_RECEIVE);
  I2C0_Disable();               // issue I2C stop signal
  return tmp;
}

/**************************************************************************************************
* Function ADXL345_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function that initializes ADXL345
* Input: Nothing
* Output: return 0 for OK; return _ACCEL_ERROR for ERROR
**************************************************************************************************/
char ADXL345_Init(void) {
  char id = 0x00;
  // Go into standby mode to configure the device.
  ADXL345_Write(0x2D, 0x00);
  id = ADXL345_Read(0x00);
  if (id != 0xE5) {
    return _ACCEL_ERROR;
  }
  else {
    ADXL345_Write(_DATA_FORMAT, 0x08);       // Full resolution, +/-2g, 4mg/LSB, right justified
    ADXL345_Write(_BW_RATE, 0x0A);           // Set 100 Hz data rate
    ADXL345_Write(_FIFO_CTL, 0x80);          // stream mode
    ADXL345_Write(_POWER_CTL, 0x08);         // POWER_CTL reg: measurement mode
    return 0x00;
  }
}

/**************************************************************************************************
* Function Accel_ReadX()
* -------------------------------------------------------------------------------------------------
* Overview: Function read X axis from accel.
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
// Read X Axis
int Accel_ReadX(void) {
  int Out_x;

  Hi(Out_x) = ADXL345_Read(_DATAX1);
  Lo(Out_x) = ADXL345_Read(_DATAX0);

  return Out_x;
}

/**************************************************************************************************
* Function Accel_ReadY()
* -------------------------------------------------------------------------------------------------
* Overview: Function read Y axis from accel.
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
int Accel_ReadY(void)
{
  int Out_y;

  Hi(Out_y) = ADXL345_Read(_DATAY1);
  Lo(Out_y) = ADXL345_Read(_DATAY0);

  return Out_y;
}

/**************************************************************************************************
* Function Accel_ReadZ()
* -------------------------------------------------------------------------------------------------
* Overview: Function read Z axis from accel.
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
int Accel_ReadZ(void)
{
  int Out_z;

  Hi(Out_z) = ADXL345_Read(_DATAZ1);
  Lo(Out_z) = ADXL345_Read(_DATAZ0);

  return Out_z;
}