/* 
 * Project name:
     MP3 Test
 * Copyright:
     (c) MikroElektronika, 2014.
 * Revision History:
     20140619(FJ):
       - initial release;
 * Description:
     This project demonstrates communication with VS1011E mp3 codec.
     Program reads one mp3 file from MMC and sends it to VS1053E for decoding and playing.
     MMC and MP3_SCI share Hardware SPI module.
* Test configuration:
     MCU:             TM4C123GH6PZ
                      http://www.ti.com/lit/ds/symlink/tm4c123gh6pz.pdf
     Dev.board:       mikromedia for Tiva
                      http://www.mikroe.com/mikromedia/tiva/
     Oscillator:      MOSC-PLL, 80.00000 MHz
     Modules:         MP3 VS1053B on-board module - ac:MP3_Codec
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 * NOTES:
     - Make sure that MMC card is properly formatted (FAT16 or just FAT)
       before testing it on this example.
     - Make sure that MMC card contains appropriate mp3 file.
 */

#include <built_in.h>
#include "MP3_Test.h"
#include "resources.h"

// TFT module connections
char TFT_DataPort at GPIO_PORTF_DATA;
sbit TFT_RST at GPIO_PORTA_DATA3_bit;
sbit TFT_BLED at GPIO_PORTD_DATA1_bit;
sbit TFT_RS at GPIO_PORTK_AHB_DATA3_bit;
sbit TFT_CS at GPIO_PORTG_DATA3_bit;
sbit TFT_RD at GPIO_PORTA_DATA6_bit;
sbit TFT_WR at GPIO_PORTA_DATA7_bit;
char TFT_DataPort_Direction at GPIO_PORTF_DIR;
sbit TFT_RST_Direction at GPIO_PORTA_DATA3_bit;
sbit TFT_BLED_Direction at GPIO_PORTD_DATA1_bit;
sbit TFT_RS_Direction at GPIO_PORTK_AHB_DATA3_bit;
sbit TFT_CS_Direction at GPIO_PORTG_DATA3_bit;
sbit TFT_RD_Direction at GPIO_PORTA_DATA6_bit;
sbit TFT_WR_Direction at GPIO_PORTA_DATA7_bit;
// End TFT module connections

// MMC Chip Select connection
sbit Mmc_Chip_Select_Direction at GPIO_PORTK_AHB_DIR0_bit;
sbit Mmc_Chip_Select           at GPIO_PORTK_AHB_DATA0_bit;
// End MMC Chip Select connections

// V1053E codec connection
sbit MP3_CS                    at GPIO_PORTG_DATA6_bit;
sbit MP3_RST                   at GPIO_PORTE_DATA6_bit;
sbit BSYNC                     at GPIO_PORTG_DATA7_bit;
sbit DREQ                      at GPIO_PORTE_DATA7_bit;

sbit MP3_CS_Direction          at GPIO_PORTG_DIR6_bit;
sbit MP3_RST_Direction         at GPIO_PORTE_DIR6_bit;
sbit BSYNC_Direction           at GPIO_PORTG_DIR7_bit;
sbit DREQ_Direction            at GPIO_PORTE_DIR7_bit;
// V1053E codec connections

/**************************************************************************************************
* Function MCU_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize MCU
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void InitMCU(){
  // Unlock PD7 and PF0 pins
  GPIO_Unlock(_GPIO_COMMIT_PIN_D7 | _GPIO_COMMIT_PIN_F0);

  // Set backlight (BLED) pin as digital output
  GPIO_Config(&GPIO_PORTD_DATA_BITS, _GPIO_PINMASK_1, _GPIO_DIR_OUTPUT, _GPIO_CFG_DIGITAL_ENABLE, 0);

  // MP3 config
  GPIO_Config(&GPIO_PORTG_DATA_BITS, _GPIO_PINMASK_6, _GPIO_DIR_OUTPUT, _GPIO_CFG_DIGITAL_ENABLE,0);
  GPIO_Config(&GPIO_PORTE_DATA_BITS, _GPIO_PINMASK_7, _GPIO_DIR_INPUT, _GPIO_CFG_DIGITAL_ENABLE,0);
  GPIO_Config(&GPIO_PORTE_DATA_BITS, _GPIO_PINMASK_6, _GPIO_DIR_OUTPUT, _GPIO_CFG_DIGITAL_ENABLE,0);
  GPIO_Config(&GPIO_PORTG_DATA_BITS, _GPIO_PINMASK_7, _GPIO_DIR_OUTPUT, _GPIO_CFG_DIGITAL_ENABLE,0);
  
  // Init TFT
  TFT_Init_ILI9341_8bit(320, 240);
  Delay_ms(100);
  TFT_BLED = 1;
}

void DrawMP3Scr(){
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("MP3  TEST", 75, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("mikromedia for Tiva", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
}

/**************************************************************************************************
* Function MC3_Init()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize MP3 VS1053B codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void MP3_Init(void)
{
  BSYNC = 1;
  MP3_CS = 1;


  // Hardware reset
  MP3_RST = 0;
  Delay_ms(10);
  MP3_RST = 1;

  while (DREQ == 0);

  MP3_SCI_Write(SCI_MODE_ADDR, 0x0800);
  MP3_SCI_Write(SCI_BASS_ADDR, 0x7A00);
  MP3_SCI_Write(SCI_CLOCKF_ADDR, 0x2000);   // default 12 288 000 Hz

  // Maximum volume is 0x00 and total silence is 0xFE.
  volume_left  = 0; //0x3F;
  volume_right = 0; //0x3F;
  MP3_Set_Volume(volume_left, volume_right);
}

/**************************************************************************************************
* Function MP3_Start()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize SPI to communicate with MP3 codec
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void MP3_Start(void)
{

  MP3_CS_Direction  = 1;               // Configure MP3_CS as output
  MP3_CS            = 1;               // Deselect MP3_CS
  MP3_RST_Direction = 1;               // Configure MP3_RST as output
  MP3_RST           = 1;               // Set MP3_RST pin

  DREQ_Direction    = 0;               // Configure DREQ as input
  BSYNC_Direction   = 1;               // Configure BSYNC as output
  BSYNC             = 0;               // Clear BSYNC
  BSYNC             = 1;               // Clear BSYNC

  // Initialize SPIC module
  SPI0_Init_Advanced(2000000, _SPI_MASTER, _SPI_8_BIT, &_GPIO_MODULE_SPI0_A245);

  TFT_BLED = 1;

  TFT_Set_Pen(CL_WHITE, 1);
  TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);

  TFT_Write_Text("1. Initializing VS1053B decoder interface", 30, 80);

  MP3_Init();
  Delay_ms(1000);
}

/**************************************************************************************************
* Function MP3_Test()
* -------------------------------------------------------------------------------------------------
* Overview: Function run the MP3 test
* Input: Nothing
* Output: test status: 0 - skiped; 1 - pass; 2 - fail
**************************************************************************************************/
void MP3_Test()
{
  unsigned long i;

  TFT_Write_Text("2. Initializing MMC_FAT", 30, 100);
  if (Mmc_Fat_Init() == 0) {
    if (Mmc_Fat_Assign(&mp3_filename, 0) ) {
      TFT_Write_Text("3. File Assigned", 30, 120);
      Mmc_Fat_Reset(&file_size);          // Call Reset before file reading,
                                          //   procedure returns size of the file

      TFT_Write_Text("4. Play audio... :)", 30, 140);
      // send file blocks to MP3 SDI
      while (file_size > BUFFER_SIZE)
      {
        for (i=0; i<BUFFER_SIZE; i++)
        {
          Mmc_Fat_Read(mp3_buffer + i);
        }
        for (i=0; i<BUFFER_SIZE/BYTES_2_WRITE; i++) {
          MP3_SDI_Write_32(mp3_buffer + i*BYTES_2_WRITE);
        }

        file_size -= BUFFER_SIZE;
     }

      // send the rest of the file to MP3 SDI
      for (i=0; i<file_size; i++)
      {
        Mmc_Fat_Read(mp3_buffer + i);
      }

      for (i=0; i<file_size; i++)
      {
        MP3_SDI_Write(mp3_buffer[i]);
      }

      TFT_Write_Text("5. Finish!", 30, 160);
    }
    else {

      TFT_Write_Text("3. File not assigned", 30, 120);
    }
  }
  else {
    TFT_Write_Text("3. MMC FAT not initialized", 30, 120);
  }
}

/**************************************************************************************************
* main function
**************************************************************************************************/

void main(){
  InitMCU();
  DrawMP3Scr();

  MP3_Start();
  MP3_Test();
}