/*
 * Project name:
     SerialFlash example (Demonstration of usage of the SerialFlash)
 * Copyright:
     (c) MikroElektronika, 2014.
 * Revision History:
     20140619 (FJ):
       - initial release;
 * Description:
      Demonstration of SerialFlash usage (chip erase, read Flash ID, simple writing and reading back a simple byte,
      simple writing and reading back an array of 16 bytes).
* Test configuration:
     MCU:             TM4C123GH6PZ
                      http://www.ti.com/lit/ds/symlink/tm4c123gh6pz.pdf
     Dev.board:       mikromedia for Tiva
                      http://www.mikroe.com/mikromedia/tiva/
     Oscillator:      MOSC-PLL, 80.00000 MHz
     Modules:         Serial Flash M25P80 on-board module - ac:serial_flash
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/

 * NOTES:
     - None.
*/

#define _DATA_ARRAY_SIZE 16
#include <built_in.h>
#include "SF_Test.h"
#include "resources.h"

// TFT module connections
char TFT_DataPort at GPIO_PORTF_DATA;
sbit TFT_RST at GPIO_PORTA_DATA3_bit;
sbit TFT_BLED at GPIO_PORTD_DATA1_bit;
sbit TFT_RS at GPIO_PORTK_AHB_DATA3_bit;
sbit TFT_CS at GPIO_PORTG_DATA3_bit;
sbit TFT_RD at GPIO_PORTA_DATA6_bit;
sbit TFT_WR at GPIO_PORTA_DATA7_bit;
char TFT_DataPort_Direction at GPIO_PORTF_DIR;
sbit TFT_RST_Direction at GPIO_PORTA_DATA3_bit;
sbit TFT_BLED_Direction at GPIO_PORTD_DATA1_bit;
sbit TFT_RS_Direction at GPIO_PORTK_AHB_DATA3_bit;
sbit TFT_CS_Direction at GPIO_PORTG_DATA3_bit;
sbit TFT_RD_Direction at GPIO_PORTA_DATA6_bit;
sbit TFT_WR_Direction at GPIO_PORTA_DATA7_bit;
// End TFT module connections

// Serial Flash Chip Select connection
sbit CS_Serial_Flash_bit              at GPIO_PORTK_AHB_DATA2_bit;
sbit CS_Serial_Flash_Direction_bit    at GPIO_PORTK_AHB_DIR2_bit;
// End Serial Flash Chip Select connection

char cSF_test_status;

/*************************************************************************************************
* Init MCU function
**************************************************************************************************/
void InitMCU(){
  // Unlock PD7 and PF0 pins
  GPIO_Unlock(_GPIO_COMMIT_PIN_D7 | _GPIO_COMMIT_PIN_F0);

  // Set backlight (BLED) pin as digital output
  GPIO_Config(&GPIO_PORTD_DATA_BITS, _GPIO_PINMASK_1, _GPIO_DIR_OUTPUT, _GPIO_CFG_DIGITAL_ENABLE, 0);

  // Set Chip Select pin as digital output
  GPIO_Config(&GPIO_PORTK_AHB_DATA_BITS, _GPIO_PINMASK_2, _GPIO_DIR_OUTPUT, _GPIO_CFG_DIGITAL_ENABLE, 0);

  TFT_BLED = 1;
}

void DrawSFScr(){
  TFT_Init_ILI9341_8bit(320,240);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("SERIAL  FLASH", 75, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("mikromedia for Tiva", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
}

/**************************************************************************************************
* Function SF_Start()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize SPI bus and serial flash
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void SF_Start(void)
{
  SPI0_Init_Advanced(20000000, _SPI_MASTER, _SPI_8_BIT, &_GPIO_MODULE_SPI0_A245);
  TFT_BLED = 1;
  TFT_Set_Font(TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Set_Pen(CL_WHITE, 1);
  TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
  SerialFlash_init();
  SerialFlash_WriteEnable();
  Delay_ms(500);
}

/**************************************************************************************************
* Function SF_Test()
* -------------------------------------------------------------------------------------------------
* Overview: Function run the serial flash test
* Input: Nothing
* Output: test status: 0 - skiped; 1 - pass; 2 - fail
**************************************************************************************************/
void SF_Test(char *test)
{
  unsigned char temp, SerialFlashID, txt[12];
  unsigned char i, success;

  // Reset error flag
  *test = 0;

  // Erase entire flash
  TFT_Write_Text("1. Erasing chip... ", 90, 65);
  SerialFlash_ChipErase();
  TFT_Write_Text("Done!", 200, 65);
 
  // Read Flash ID
  SerialFlashID = SerialFlash_ReadID();
  ByteToStr(SerialFlashID, txt);
  TFT_Write_Text("2. Flash ID:", 90, 85);
  TFT_Write_Text(txt, 170, 85);
  Delay_ms(500);

  // Write and read back a single byte
  temp = 221;
  TFT_Write_Text("3. Writting:", 90, 105);
  ByteToStr(temp, txt);
  TFT_Write_Text(txt, 170, 105);
  SerialFlash_WriteByte(temp, 0x123456);

  TFT_Write_Text("4. Reading: ", 90, 125);
  temp = SerialFlash_ReadByte(0x123456);
  ByteToStr(temp, txt);
  TFT_Write_Text(txt, 170, 125);
  Delay_ms(500);

  // Write the repetitive pattern of data to fill the first 4K of memory
  // and then read the entire flash again and check the data correctness
  TFT_Write_Text("5. Writing array...", 90, 145);
  SerialFlash_WriteArray(0x0000, &write_array, _DATA_ARRAY_SIZE);
  Delay_ms(500);
  
  TFT_Write_Text("6. Reading array...", 90, 165);
  SerialFlash_ReadArray(0x0000, &read_array, _DATA_ARRAY_SIZE);
  Delay_ms(500);
  success = 1;
  
  for (i = 0; i<_DATA_ARRAY_SIZE; i++)
  {
    if (read_array[i] != write_array[i])
    {
      success = 0;
      break;
    }
  }
  
  if (success)
  {
    TFT_Write_Text("7. Success - Full match!", 90, 185);
    *test = 1;
  }
  else
  {
    TFT_Write_Text("7. Failed", 90, 185);
    *test = 2;
  }
}

/**************************************************************************************************
* main function
**************************************************************************************************/
void main(){
  InitMCU();
  DrawSFScr();
  SF_Start();
  SF_Test(&cSF_test_status);
}